"""注意力机制模块

包含 CBAM 和 EEG 专用注意力机制。
"""

from . import attn_utils
from .CBAM import CBAM1D, ChannelAttention1D, SpatialAttention1D
from .eeg import EEGChannelAttention, SpectralAttention, TemporalAttention
from .efficient import LinformerAttention, PerformerAttention, SlidingWindowAttention
from .multihead import CrossAttention, MultiHeadAttention, SelfAttention

__all__ = [
    # CBAM 注意力
    'CBAM1D',
    'ChannelAttention1D',
    'SpatialAttention1D',
    # EEG 专用注意力
    'EEGChannelAttention',
    'TemporalAttention',
    'SpectralAttention',
    # Transformer-based Attention
    'MultiHeadAttention',
    'SelfAttention',
    'CrossAttention',
    'LinformerAttention',
    'SlidingWindowAttention',
    'PerformerAttention',
    # 注意力工具函数
    'attn_utils',
]
