"""
zyplib is a self-owned Python library for academic research on deep learning with biosignals developed by yp.z.

It provides a collection of tools and modules for data processing, model training, evaluation, and visualization.

zyplib/
├── __SKILL__/
├── annotation/
├── artifact/
├── dataset/    # Load dataset, do spliting, build dataloader etc.
├── experiment/  # Launch and manage deep learning experiments.
├── feature/  # Extract EEG signal features.
├── io/     # File I/O for EEG data file.
├── nn/     # Predefined torch modules.
├── paradigm/  # Some DL algorithms, e.g. GAN, Evidential Deep Learning etc.
├── preprocessing/  # Preprocess EEG signals, e.g., filtering, segmentation.
├── signal/
├── train/  # Model train framework.
├── transformation/  # Data augmentation, mixup etc.
├── utils/  # Bunch of useful tools.
├── vis/  # Plotting.
├── __init__.py
├── cli.py
└── import_tools.py

zyplib provides with Agent SKILL for usage.
Run `python -m zyplib.cli list_skills` to list all available skills.
"""


# from . import dataset, feature, io, nn, preprocessing, signal, train, utils, vis
from ._config import config  # 初始化
from .import_tools import import_lazy, import_relative  # 导入工具

__all__ = [
    'config',
    'import_lazy',
    'import_relative',
    # 'fabric',
    # 'dataset',
    # 'feature',
    # 'io',
    # 'nn',
    # 'preprocessing',
    # 'signal',
    # 'train',
    # 'utils',
    # 'vis',
]
