from typing import Literal

import numpy as np
from numpy._typing import NDArray


def padding(
    X: NDArray,
    padtype: Literal['zero', 'odd', 'even'],
    padlen: int | None = None,
    axis=-1,
):
    """对信号进行填充


    Parameters
    ----------
    - `X` : `NDArray`
        - 输入信号
    - `padtype` : `zero | odd | even`
        - 填充类型
    - `padlen` : `int | None`, optional
        - 填充长度，None 则为 scipy 默认 padlen
    - `axis` : `int`, optional
        - 填充轴，-1 表示最后一个轴
    """
