from .auto_encoder import (
    AutoEncoder,
    AutoEncoderBase,
    BetaVAE,
    ConditionalVAE,
    VariationalAutoEncoder,
)

from .gan import (
    LSGAN,
    WGAN,
    WGANGP,
    GANBase,
    GradientRegularization,
    HingeGAN,
    RaGAN,
    SpectralNormMixin,
    StandardGAN,
)


__all__ = [
    # AutoEncoder paradigms
    'AutoEncoder',
    'AutoEncoderBase',
    'BetaVAE',
    'ConditionalVAE',
    'VariationalAutoEncoder',
    # GAN paradigms
    'GANBase',
    'StandardGAN',
    'WGAN',
    'WGANGP',
    'LSGAN',
    'HingeGAN',
    'RaGAN',
    'GradientRegularization',
    'SpectralNormMixin',
]
