from typing import Literal
from zyplib.nn.modules.base import BaseModule


class ResampleTo(BaseModule):
    """时间序列重采样


    假定输入样本 [N, C, T]

    本模块的作用是将时间维度 T 重采样到指定的长度 out_samples
    """

    def __init__(
        self, out_samples: int, method: Literal['interpolate', 'adaptive_pool'], t_dim=-1
    ): ...
