# zyplib/utils/convert.py
"""
由于历史原因，这个模块曾经包含了一个 to_mne_raw 函数；
现在这个函数被移到了 zyplib/io/_base.py 中。
这里保留了一个属性访问的接口，以便于向后兼容。
"""

_store = {}


def __getattr__(name: str):
    if name == 'to_mne_raw':
        if 'to_mne_raw' not in _store:
            from zyplib.io._base import to_mne_raw

            _store['to_mne_raw'] = to_mne_raw
        return _store['to_mne_raw']
    raise AttributeError(f'module {__name__!r} has no attribute {name!r}')


def __dir__():
    return sorted(list(globals().keys()) + ['to_mne_raw'])
