"""训练基础设施模块

提供独立的训练相关工具类，可以与 Callback 组合使用，
也可以在自定义训练循环中独立使用。
"""

from zyplib.train.infra.early_stop import EarlyStoppingMonitor
from zyplib.train.infra.fabric_log import LogScalarManager
from zyplib.train.infra.print_info import TrainInfoPrinter
from zyplib.train.infra.save_ckpt import TopKCheckpointManager

__all__ = [
    'EarlyStoppingMonitor',
    'LogScalarManager',
    'TrainInfoPrinter',
    'TopKCheckpointManager',
]
